-- Renders a walkthrough animation and saves its frames or its camera and target positions to disk.
-- To use it, select the desired render target and then run the script.
-- 
-- @description Renders a walkthrough animation and saves its frames on disk.
-- @author      Thomas Loockx, Roeland, Jason Grimes, Mark Bassett
-- @shortcut    alt + w (not working)
-- @shortcut    ctrl + w (not working)
-- @shortcut    ctrl + 3

-- @version     1.0
--

-- local version = "1.0"		Initial code by Thomas, Roeland & Jason
--					This code based on their release for Octane v1.25 it is
--					thus missing several subsequent refinements possible at v1.29 it does
--					incorporate Jason's preview feature. It has had extensive renaming of 
--					controls and variables for personal clarity, will probably do more in this area.

--					Known Issues:
--					desired short cut key combo does not seem to work
--					double error messaging, initial trap does not stop subsequent error
--					save to positions file feature is not yet implemented.

-- local version = "1.01"		Ammended to accomodate Texture Environment Nodes.
-- local version = "1.02"		Fixed UI enabling/disabling.
--					Added animation of camera settings
--					Changed some UI terminology
-- local version = "1.03"		Added ability to combine multiple camera moves.	


local version = "1.03 - for Octane Standalone V1.31"





-- Pause the current renderer
octane.render.pause()

--Global Variables
numCameraMove = 0
blnMultipleMoves = true

-- directory for required files and defaults
filePath = "Octane Render Walkthrough/"

-- load Octane ui componets
ui = require(filePath.."uiWalkthrough")

-- load the gui
require(filePath.."Walkthrough_gui")

-- load the helper functions
require(filePath.."Walkthrough_helpers")

-- load the callback functions
require(filePath.."Walkthrough_callbacks")

-- load the animation functions
require(filePath.."Walkthrough_anim")

-- form/window that holds all components
local turntableWindow = octane.gui.create
{ 
    type     = octane.gui.componentType.WINDOW,
    text     = "Octane Render Walkthrough V"..version,
    children = { layoutGrp },
    width    = layoutGrp:getProperties().width,
    height   = layoutGrp:getProperties().height,
}


-- callback handling the GUI elements
local function guiCallback(component, event)

    local index =-1

	if component == sldDuration then 

	-- Update the #frames
	local frames = math.ceil(sldDuration:getProperties().value * sldFrameRate:getProperties().value)
	sldNumberOfFrames:updateProperties{ value = frames }


	elseif component == chkMultipleMoves then
		
		if chkMultipleMoves:getProperties().checked == false then

			blnMultipleMoves = false
			numCameraMove = 0
			lblCameraMoveNumber:updateProperties{text = "Last Camera Move: "}
			lblCameraMoveNumber:updateProperties{ enable = false } 
			btnUndoLastCameraMove:updateProperties{ enable = false }
			btnPreviousRender:updateProperties{ enable = false }

		elseif  chkMultipleMoves:getProperties().checked == true then

			blnMultipleMoves = true
			numCameraMove = 0
			lblCameraMoveNumber:updateProperties{text = "Last Camera Move: "}
			lblCameraMoveNumber:updateProperties{ enable = true }
			btnUndoLastCameraMove:updateProperties{ enable = true }
			--btnPreviousRender:updateProperties{ enable = true}

		end


	-- Undo the last camera move
	elseif component == btnUndoLastCameraMove then

		print("Undo attempted by user")
		print(" Undo Camera Position: ",posXEndCameraPosition[1],posXEndCameraPosition[2],posXEndCameraPosition[3])
		print(" Undo Camera Position: ",posXEndCameraTarget[1],posXEndCameraTarget[2],posXEndCameraTarget[3])
		print(" Undo Camera Position: ",numXEndFieldOfView)
		print(" Undo Camera Position: ",numXEndNearClipDepth)

		--Set variables back to previous values
		posEndCameraPosition 	= 	posXEndCameraPosition
		posEndCameraTarget 	= 	posXEndCameraTarget
		numEndFieldOfView 	= 	numXEndFieldOfView
		numEndNearClipDepth 	= 	numXEndNearClipDepth

		numCameraMove = numCameraMove -1
		lblCameraMoveNumber:updateProperties{text = "Last Camera Move: "  .. tostring(numCameraMove)}
		btnUndoLastCameraMove:updateProperties{ enable = false}


	elseif component == sldFrameRate then 

		-- if the frame rate changes, update the #frames
		local frames = math.ceil(sldDuration:getProperties().value * sldFrameRate:getProperties().value)
		sldNumberOfFrames:updateProperties{ value = frames }


	elseif component == sldNumberOfFrames then 

		-- if the #frames changes, update the duration of the animation
		local duration = sldNumberOfFrames:getProperties().value / sldFrameRate:getProperties().value
		sldDuration:updateProperties{ value = duration }


    elseif component == btnPreview then

	print("Previewing camera move.")

	-- create a fresh copy just before we start rendering.
		SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE = getSceneCopy()

	if blnMultipleMoves == true then

		if numCameraMove == 0 then

			-- Initialize global variables to store last settings from each camera move
			posEndCameraPosition = CAM_NODE:getPinValue(octane.P_POSITION)
			posEndCameraTarget = CAM_NODE:getPinValue(octane.P_TARGET)
			numEndFieldOfView = CAM_NODE:getPinValue(octane.P_FOV)
			numEndNearClipDepth = CAM_NODE:getPinValue(octane.P_NEAR_CLIP_DEPTH)

			-- Initialize global variables to store 2nd to last settings from each camera move
			posXEndCameraPosition = CAM_NODE:getPinValue(octane.P_POSITION)
			posXEndCameraTarget = CAM_NODE:getPinValue(octane.P_TARGET)
			numXEndFieldOfView = CAM_NODE:getPinValue(octane.P_FOV)
			numXEndNearClipDepth = CAM_NODE:getPinValue(octane.P_NEAR_CLIP_DEPTH)

		else

			if OUT_PATH ~= nil then

				local rv = showOkCancel("Warning","You did not output your previous 'Camera Move'.")

				if rv == 1 then

					--Keep going

				elseif rv == 2 then

        				cancelRender()

					--Exit function
					do return end

				end

		end


			-- Populate global variables to store 2nd to last settings from each camera move
			posXEndCameraPosition = posEndCameraPosition
			posXEndCameraTarget = posEndCameraTarget
			numXEndFieldOfView = numEndFieldOfView
			numXEndNearClipDepth = numEndNearClipDepth

   			--local target   	= CAM_NODE:getPinValue(octane.P_TARGET)
    			--local position 	= CAM_NODE:getPinValue(octane.P_POSITION)
    			--local viewDir  	= CAM_NODE.octane.vec.sub(target, position)
    			--local dtt   		= CAM_NODE.length(viewDir)
			--dtt 		= round(dtt,3)

			--lblInitialFieldOfView:updateProperties{ text = "Field of view: " .. tostring(CAM_NODE:getPinValue(octane.P_FOV))}	
			--lblInitialNearClip:updateProperties{ text = "Near clip depth: " .. tostring(CAM_NODE:getPinValue(octane.P_NEAR_CLIP_DEPTH))}	
			--lblInitialDistanceToTarget:updateProperties{ text = "Distance to target: " .. tostring(dtt)}


	end

	numCameraMove = numCameraMove + 1

	if numCameraMove >0 and OUT_PATH ~= nil then 
		btnPreviousRender:updateProperties{ enable = true }
	elseif numCameraMove < 1 then
		btnPreviousRender:updateProperties{ enable = false }
	end

	lblCameraMoveNumber:updateProperties{text = "Last Camera Move: "  .. tostring(numCameraMove)}
	btnUndoLastCameraMove:updateProperties{ enable = true}

end

	stopButton:updateProperties{ enable = true }
	startRender(SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE, OUT_PATH, false)



    elseif component == btnOutputFrames then 

        -- choose an output file
        local ret = octane.gui.showDialog
        {
            type      = octane.gui.dialogType.FILE_DIALOG,
            title     = "Provide a seed name for the frame sequence.",
            wildcards = "*.png",
            save      = true,
        }

        -- if a file is chosen
        if ret.result ~= "" then

	renderButton:updateProperties{ enable = true } 
	if numCameraMove >1 then btnPreviousRender:updateProperties{ enable = true } end
	fileEditorFrames:updateProperties{ text = ret.result }
	OUT_PATH = ret.result

        else

	renderButton:updateProperties{ enable = false } 
	btnPreviousRender:updateProperties{ enable = false } 
	fileEditorFrames:updateProperties{ text = "" }
	OUT_PATH = nil

        end


    elseif component == btnOutputPositions then 

        -- choose an output file
        local ret = octane.gui.showDialog
        {
            type      = octane.gui.dialogType.FILE_DIALOG,
            title     = "Choose the output file",
            wildcards = "*.png",
            save      = true,
        }

        -- if a file is chosen
        if ret.result ~= "" then

	renderButton:updateProperties{ enable = true }
	btnPreviousRender:updateProperties{ enable = true } 
	fileEditorPositions:updateProperties{ text = ret.result }
	OUT_PATH = ret.result

        else

	renderButton:updateProperties{ enable = false } 
	btnPreviousRender:updateProperties{ enable = false } 
            fileEditorPositions:updateProperties{ text = "" }
            OUT_PATH = nil

        end


    elseif component == renderButton then 

	-- create a fresh copy just before we start rendering.
		SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE = getSceneCopy()

	if blnMultipleMoves == true then

		if numCameraMove == 0 then

			-- Initialize global variables to store last settings from each camera move
			posEndCameraPosition = CAM_NODE:getPinValue(octane.P_POSITION)
			posEndCameraTarget = CAM_NODE:getPinValue(octane.P_TARGET)
			numEndFieldOfView = CAM_NODE:getPinValue(octane.P_FOV)
			numEndNearClipDepth = CAM_NODE:getPinValue(octane.P_NEAR_CLIP_DEPTH)

			-- Initialize global variables to store 2nd to last settings from each camera move
			posXEndCameraPosition = CAM_NODE:getPinValue(octane.P_POSITION)
			posXEndCameraTarget = CAM_NODE:getPinValue(octane.P_TARGET)
			numXEndFieldOfView = CAM_NODE:getPinValue(octane.P_FOV)
			numXEndNearClipDepth = CAM_NODE:getPinValue(octane.P_NEAR_CLIP_DEPTH)

		else

			-- Initialize global variables to store 2nd to last settings from each camera move
			posXEndCameraPosition = posEndCameraPosition
			posXEndCameraTarget = posEndCameraTarget
			numXEndFieldOfView = numEndFieldOfView
			numXEndNearClipDepth = numEndNearClipDepth


		end

		numCameraMove = numCameraMove + 1
		lblCameraMoveNumber:updateProperties{text = "Last Camera Move: "  .. tostring(numCameraMove)}
		btnUndoLastCameraMove:updateProperties{ enable = true}

end

		stopButton:updateProperties{ enable = true }
		startRender(SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE, OUT_PATH, true)


    elseif component == btnPreviousRender then

		--showMessage(tostring(numCameraMove))

		-- create a fresh copy just before we start rendering.
		SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE = getSceneCopy()

		if numCameraMove >0 then

		--Set back to previous values
		posEndCameraPosition 	= 	posXEndCameraPosition
		posEndCameraTarget 	= 	posXEndCameraTarget
		numEndFieldOfView 	= 	numXEndFieldOfView
		numEndNearClipDepth 	= 	numXEndNearClipDepth

		stopButton:updateProperties{ enable = true }
		startRender(SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE, OUT_PATH, true)

		end


    elseif component == renderTargetDropDown then -- not used
 
        -- local renderTarget   = renderTargetDropDown:getProperties().selectedItem
        -- A render target has been choosen in the drop down, now 
        -- see if it matches the current selected target node.  If they match
        -- then do nothing.  If they do not match then set RT_NODE and CAM_NODE to
        -- new nodes.

        if renderTarget ~= "" and RT_NODE:getProperties().name ~= renderTarget then

            -- find the choosen render target node, and copy the scene
            SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE = getSceneCopy(renderTarget)

            -- re-initialize the UI
            initGui(CAM_NODE)

        end


    elseif component == stopButton then

        cancelRender()


    elseif component == exitButton then

        cancelRender()
        turntableWindow:closeWindow()


    elseif component == turntableWindow then

        -- when the window closes, cancel rendering
        if event == octane.gui.eventType.WINDOW_CLOSE then

            cancelRender()


        end

    end

end


-- Global variable that holds the output path
	OUT_PATH = nil

-- Get the render target sun and camera in global variables
	SCENE_GRAPH, RT_NODE, CAM_NODE, SUN_DIR_NODE = getSceneCopy()
	selectedTarg = RT_NODE:getProperties().name

-- Initialize the render targets combo box
	--local renderTargets = getAllrendertargets()
	--renderTargetDropDown:updateProperties { items = renderTargets, selectedItem = selectedTarg }


-- initialize the UI
initGui(CAM_NODE)
renderButton:updateProperties{ enable = false }
btnPreviousRender:updateProperties{ enable = false }

-- hookup the callback with all the GUI elements
sldDuration			:updateProperties		{ callback = guiCallback }
sldFrameRate			:updateProperties		{ callback = guiCallback }
sldNumberOfFrames		:updateProperties		{ callback = guiCallback }
chkMultipleMoves			:updateProperties		{ callback = guiCallback }
btnUndoLastCameraMove		:updateProperties		{ callback = guiCallback }
btnOutputFrames			:updateProperties		{ callback = guiCallback }
btnPreview			:updateProperties		{ callback = guiCallback }
btnPreviousRender			:updateProperties		{ callback = guiCallback }
renderButton			:updateProperties		{ callback = guiCallback }
stopButton			:updateProperties		{ callback = guiCallback }
exitButton				:updateProperties		{ callback = guiCallback }
renderTargetDropDown		:updateProperties		{ callback = guiCallback }
defaultLoadButton			:updateProperties		{ callback = defaultLoadCallback }
defaultSaveButton			:updateProperties		{ callback = defaultSaveCallback }
defaultResetButton			:updateProperties		{ callback = defaultResetCallback }
turntableWindow			:updateProperties		{ callback = guiCallback }


-- the script will pause here until the window closes
turntableWindow:showWindow()



-- Continue the previous renderer
-- octane.render.continue()