----------------------------------------------------------------------------------------------------------------
-- Walkthrough Animation Functions
----------------------------------------------------------------------------------------------------------------


-- Copies of the original scene graph, camera node and the rendertarget. Prevents modification of the original scene.


local outputfilename = ""

function getSceneCopy()

    -- Get the selected render target
    local selectedRt = octane.project.getSelection()[1]

    -- Find the index of the selected render target node
    local idx = -1

    for i, item in ipairs(octane.nodegraph.getRootGraph():getOwnedItems()) do

        if item == selectedRt then

            idx = i

            break

        end

    end


    -- See if we could find the node
    if idx == -1 then showError("Walk Through Error!","No render target selected", true) end
    
    -- Create a full copy of the project so that we don't modify the original project
    local copyScene = octane.nodegraph.createRootGraph("Project Copy")
    copies = copyScene:copyFrom(octane.nodegraph.getRootGraph():getOwnedItems())
    copyRt = copies[idx]
    

    -- Check if the copied node is a render target
    if not copyRt or copyRt:getProperties().type ~= octane.NT_RENDERTARGET then

        showError("Walkthrough Error!","No 'Render Target' selected", true)


    end


    -- Check if a thin lens camera is connected to the render target
    local copyCam = copyRt:getConnectedNode(octane.P_CAMERA)

    if not copyCam or copyCam :getProperties().type ~= octane.NT_CAM_THINLENS then

        showError("Walkthrough Error!", "A 'Thinlens' camera node must be connected to the render target", true)


    end


    -- Check if an environment node is connected to the render target
    local copyEnv = copyRt:getConnectedNode(octane.P_ENVIRONMENT)
    copySunDir = nil

    if not copyEnv then

        showError("Walkthrough Error!", "A 'Daylight' or 'Texture' environment node must be connected to the render target", true)


    else

    -- Check if a sun direction node and it is connected to the daylight environment

	if copyEnv:getProperties().type == octane.NT_ENV_DAYLIGHT then

   		copySunDir = copyEnv:getConnectedNode(octane.P_SUN_DIR)

    		if not copySunDir or copySunDir:getProperties().type ~= octane.NT_SUN_DIRECTION then

        			showError("Walkthrough Error!", "No sun direction connected to the daylight environment", true)

   		end
	else

	sldSolarStartHour	:updateProperties{ enable = false }
	sldSolarEndHour		:updateProperties{ enable = false }

	end

    end


    -- Find all render targets
    --local renderTargets = octane.nodegraph.getRootGraph():findNodes(octane.NT_RENDERTARGET)
    --local renTargList = {}

    --for i, item in ipairs(renderTargets) do

        -- renTargList[item:getProperties().name] = item
        --renTargList[#renTargList + 1] = item:getProperties().name

    --end


    return copyScene, copyRt, copyCam , copySunDir --, renTargList

end




-- Set Camera Animator
function setCamAnimator(camNode, CamHorizontalRotAmount, angRotationOffset, numDistanceToTarget, nbFrames)

	if blnMultipleMoves == true then

		if posEndCameraPosition ~= nil then camNode:setPinValue(octane.P_POSITION, posEndCameraPosition, false) end
		if posEndCameraTarget ~= nil then camNode:setPinValue(octane.P_TARGET, posEndCameraTarget,false) end
		if numEndFieldOfView ~= nil then camNode:setPinValue(octane.P_FOV, numEndFieldOfView ,false) end
		if numEndNearClipDepth ~= nil then camNode:setPinValue(octane.P_NEAR_CLIP_DEPTH, numEndNearClipDepth,false) end

	end

    	-- Get UI and Node settings
	local vecInitialCameraUp			= camNode:getPinValue(octane.P_UP)
	local posInitialCameraTarget			= camNode:getPinValue(octane.P_TARGET)
	local posInitialCameraPosition		= camNode:getPinValue(octane.P_POSITION)

	local vecInitialViewDirection			= octane.vec.normalized(octane.vec.sub(posInitialCameraTarget, posInitialCameraPosition))
   	local posInitialCameraRight   			= octane.vec.cross(octane.vec.normalized(vecInitialViewDirection), vecInitialCameraUp)
	local numInitialDistanceToTarget 		= octane.vec.length(octane.vec.sub(posInitialCameraPosition,posInitialCameraTarget))

	local vecCurrentCameraUp       		= camNode:getPinValue(octane.P_UP)
	--local posCurrentCameraTarget   		= camNode:getPinValue(octane.P_TARGET)
	--local posCurrentCameraPosition 		= camNode:getPinValue(octane.P_POSITION)
	--local vecCurrentViewDirection     		= octane.vec.normalized(octane.vec.sub(posInitialCameraTarget, posInitialCameraPosition))
   	--local vecCurrentCameraRight    		= octane.vec.cross(octane.vec.normalized(vecInitialViewDirection), vecInitialCameraUp)

	local numFowardBackAmount			= sldCameraFowardBack:getProperties().value	
	local numCamOrbitUpDownAmount  		= sldCameraOrbitUpDown:getProperties().value

	local numCamRotateLeftRightAmount		= -1* sldTargetLeftRight:getProperties().value
	local numCamRotateUpDownAmount		= -1* sldTargetUpDown:getProperties().value

	local numCamPanLeftRightAmount		= sldCameraPanLeftRight:getProperties().value
	local numCamPanUpDownAmount		= -1* sldCameraPanUpDown:getProperties().value

	local numSolarStartHour			= sldSolarStartHour:getProperties().value
	local numSolarEndHour			= sldSolarEndHour:getProperties().value

	local numInitialFieldOfView			= camNode:getPinValue(octane.P_FOV)
	local numInitialNearClipDepth			= camNode:getPinValue(octane.P_NEAR_CLIP_DEPTH)
	local numCurrentFieldOfView			= camNode:getPinValue(octane.P_FOV)
	local numCurrentNearClipDepth		= camNode:getPinValue(octane.P_NEAR_CLIP_DEPTH)
	local numFinalFieldOfView			= sldFieldOfView:getProperties().value
	local numFinalNearClipDepth			= sldNearClipDepth:getProperties().value

	--Create zero vectors
	local vecViewDirectionHorizontalRotation	= octane.vec.scale(vecInitialViewDirection,0)
	local vecViewDirectionVerticalRotation		= octane.vec.scale(vecInitialViewDirection,0)

	local posCameraTargetHorizontal		= octane.vec.scale(posInitialCameraTarget , 0) 
	local posCameraTargetVertical		= octane.vec.scale(posInitialCameraTarget , 0)

	local vecPanHorizontal			= octane.vec.scale(posInitialCameraTarget , 0) 
	local vecPanVertical			= octane.vec.scale(posInitialCameraTarget , 0) 	
	local vecPan				= octane.vec.scale(posInitialCameraTarget , 0) 	

	local vecViewRotationDirection		= octane.vec.scale(vecInitialViewDirection, 0) 
	local posRotatedCamTarget			= octane.vec.scale(posInitialCameraTarget , 0) 

	local posPannedCameraPosition 		= octane.vec.scale(posInitialCameraPosition , 0) 
	local posPannedCameraTarget		= octane.vec.scale(posInitialCameraTarget , 0) 

	local vecCurrentViewDirection		= octane.vec.scale(vecInitialViewDirection  , 0) 
	local posCurrentCameraPosition		= octane.vec.scale(posInitialCameraPosition , 0) 
	local posCurrentCameraTarget		= octane.vec.scale(posInitialCameraTarget , 0) 

	local vecCurrentCameraRight			= octane.vec.scale(posInitialCameraRight , 0) 
	local posOrbitedCamera			= posInitialCameraPosition


	numTargetOffset = targetSlider:getProperties().value
	numDistanceToTarget = octane.vec.length(octane.vec.sub(posInitialCameraPosition,posInitialCameraTarget))
	numDistanceToTarget = numDistanceToTarget + numTargetOffset
	numInitialDistanceToTarget = numInitialDistanceToTarget + numTargetOffset


 
    	-- NOTE: (targetposition = cameraposition + viewdirection)
	posInitialCameraTarget = octane.vec.add(posInitialCameraPosition , octane.vec.scale(vecInitialViewDirection, numDistanceToTarget ))

	local blnOrbiting		= false
	local blnFowardBack	= false
	local blnRotating		= false
	local blnPanning		= false
	local blnInvertNCD		= chkInvert:getProperties().checked


	if blnInvertNCD == true then

		numCurrentNearClipDepth = numFinalNearClipDepth

		local tmp = numInitialNearClipDepth

		numInitialNearClipDepth = numFinalNearClipDepth

		numFinalNearClipDepth = tmp

	end


	--Initialize flags
	if numCamOrbitUpDownAmount  ~= 0 or CamOrbitLeftRightAmount  ~= 0 then
		blnOrbiting = true
	end


	if numFowardBackAmount ~= 0  then
		blnFowardBack = true
	end


	if numCamRotateLeftRightAmount ~= 0 or numCamRotateUpDownAmount  ~= 0 then
		blnRotating = true
	else
		blnRotating = false	
	end


	if numCamPanLeftRightAmount  ~= 0 or numCamPanUpDownAmount  ~= 0 then
		blnPanning = true
	else
		blnPanning = false
	end


	-- Tables to store results of calculations
	local tblCameraPositions = {}
	local tblCameraTargets ={}
    	local tblCameraUp ={}
	local tblFieldOfView={}
	local tblNearClipDepth={}

	local function ease(x)

		return ((x) * (x) * (3 - 2 * (x)))

	end

    -- Calculate the new camera position for each frame
    -- NOTE: targetposition = cameraposition + viewdirection, NOTE: camposition = targetposition - viewdirection
	
    for i = 0, nbFrames - 1 do

	local V = i/nbFrames
	
	-- Orbit the camera by adjusting the Camera Position
	------------------------------------------------------------------------------------------------------------
	print("-----------------------------------------------------------------------------")
	print("Frame " .. i .. " of Camera Move "..(numCameraMove))
	outputfilename = "Camera Move "..(numCameraMove).." - " .. "Frame " .. i
	-- print(outputfilename)

 	if blnOrbiting == true then

        	-- Calculate the horizontal angle of rotation for this frame relative to the original view direction
		local angHorizontalOrbitIncrement = math.rad( (ease(V)) * CamHorizontalRotAmount)

        	-- Calculate the vertical angle of rotation for this frame
		local angVerticalOrbitIncrement = math.rad( (ease(V)) * numCamOrbitUpDownAmount )


        	-- Rotate the viewing direction around the camera's up vector to orbit left or right
		vecCurrentViewDirection = octane.vec.rotate(vecInitialViewDirection, vecInitialCameraUp, angHorizontalOrbitIncrement)
		print(" vecCurrentViewDirection1",vecCurrentViewDirection[1],vecCurrentViewDirection[2],vecCurrentViewDirection[3])

-- Start Broken

        	-- Rotate the viewing direction around the camera's right vector to orbit up or down

		vecCurrentCameraRight = octane.vec.cross(octane.vec.normalized(vecCurrentViewDirection), vecCurrentCameraUp)

		if octane.vec.length(vecCurrentCameraRight) > 0 then

			vecCurrentViewDirection = octane.vec.rotate(vecCurrentViewDirection, vecCurrentCameraRight , angVerticalOrbitIncrement)

		else

			vecCurrentCameraUp = {0,0,-1}
			vecCurrentCameraRight = octane.vec.cross(octane.vec.normalized(vecCurrentViewDirection), vecCurrentCameraUp)
			vecCurrentViewDirection = octane.vec.rotate(vecCurrentViewDirection, vecCurrentCameraRight , angVerticalOrbitIncrement)

		end
		
		print(" vecCurrentViewDirection2",vecCurrentViewDirection[1],vecCurrentViewDirection[2],vecCurrentViewDirection[3])

	end

-- End Broken

	if blnFowardBack == true then

        		-- Adjust the target distance to move in or out
        		if numFowardBackAmount ~= 0 then

			numDistanceToTarget = numInitialDistanceToTarget +  ((ease(V)) *numFowardBackAmount)
			

				if numDistanceToTarget < 0 then

 					numDistanceToTarget = 0

 				end

        		end

	end


        -- Scale the new view direction by the target distance
	    vecCurrentViewDirection = octane.vec.scale(vecCurrentViewDirection, numDistanceToTarget)
		print(" vecCurrentViewDirection3",vecCurrentViewDirection[1],vecCurrentViewDirection[2],vecCurrentViewDirection[3])


        -- Calculate the new camera position,  (NOTE: camposition = targetposition - viewdirection )
	    posOrbitedCamera = octane.vec.sub(posInitialCameraTarget, vecCurrentViewDirection)

		print(" posOrbitedCamera",posOrbitedCamera[1],posOrbitedCamera[2],posOrbitedCamera[3])
		print(" posInitialCameraPosition",posInitialCameraPosition[1],posInitialCameraPosition[2],posInitialCameraPosition[3])
		print(" posInitialCameraTarget",posInitialCameraTarget[1],posInitialCameraTarget[2],posInitialCameraTarget[3])
		print(" vecInitialViewDirection",vecInitialViewDirection[1],vecInitialViewDirection[2],vecInitialViewDirection[3])


	-- Swivel the camera, by adjusting the Target Position
	------------------------------------------------------------------------------------------------------------
	if blnRotating == true then

		-- Rotate the Camera about its vertical axis
		if numCamRotateLeftRightAmount	  ~= 0 then

			-- Rotate the camera left/right
			local angHorizontalRotate = math.rad( (ease(V)) * numCamRotateLeftRightAmount - angRotationOffset )
            			vecViewDirectionHorizontalRotation = octane.vec.rotate(vecInitialViewDirection, vecInitialCameraUp, angHorizontalRotate )

		end


		-- Rotate the camera about its horizontal axis
		if numCamRotateUpDownAmount  ~= 0 then

			-- Rotate the camera Up/Down
			local angVerticalRotate = math.rad( (ease(V)) * numCamRotateUpDownAmount )
            			vecViewDirectionVerticalRotation = octane.vec.rotate(vecInitialViewDirection, posInitialCameraRight, angVerticalRotate )

		end


		-- Calculate the combined view direction
		vecViewRotationDirection   = octane.vec.add(vecViewDirectionHorizontalRotation , vecViewDirectionVerticalRotation )
			print(" vecViewRotationDirection",vecViewRotationDirection[1],vecViewRotationDirection[2],vecViewRotationDirection[3])

		-- Rotate about current camera position,  ( targetposition = cameraposition + viewdirection )
		posRotatedCamTarget = octane.vec.add(posOrbitedCamera , vecViewRotationDirection )
			print(" posRotatedCamTarget",posRotatedCamTarget[1],posRotatedCamTarget[2],posRotatedCamTarget[3])


	end

	
	--Pan the camera by adjusting the Camera and the Target Position
	------------------------------------------------------------------------------------------------------------
	if blnPanning == true then

		-- Pan the view horizontally, move the camera
		if numCamPanLeftRightAmount  ~= 0 then

			-- move camera left ot right
			vecCurrentCameraRight = octane.vec.cross(octane.vec.normalized(vecCurrentViewDirection), vecInitialCameraUp)
			vecPanHorizontal = octane.vec.scale(vecCurrentCameraRight, (ease(V)) * numCamPanLeftRightAmount)

		end


		-- Pan the view vertically, move the camera
		if numCamPanUpDownAmount  ~= 0 then

			--move camera left ot right
			vecPanVertical = octane.vec.scale(vecInitialCameraUp, (ease(V)) * numCamPanUpDownAmount)

		end


		-- Calculated combined panning movement
		vecPan = octane.vec.add(vecPanHorizontal , vecPanVertical )
			print(" vecPan",vecPan[1],vecPan[2],vecPan[3])


		-- Move current camera position by pan vector amount
		posPannedCameraPosition = octane.vec.add(posCurrentCameraPosition , vecPan )
			print("posPannedCameraPosition",posPannedCameraPosition[1],posPannedCameraPosition[2],posPannedCameraPosition[3])


		-- Move current camera target by pan vector amount
		posPannedCameraTarget = octane.vec.add(posInitialCameraTarget , vecPan)
			print(" posPannedCameraTarget",posPannedCameraTarget[1],posPannedCameraTarget[2],posPannedCameraTarget[3])


	end


	--Put it all together
	------------------------------------------------------------------------------------------------------------
	if blnOrbiting == true then
		
		-- Current camera position should be zero at this point
		posCurrentCameraPosition = octane.vec.add(posCurrentCameraPosition,posOrbitedCamera)
			print(" posCurrentCameraPosition1",posCurrentCameraPosition[1],posCurrentCameraPosition[2],posCurrentCameraPosition[3])

	end


	if blnRotating == true or blnPanning == true then

		if blnPanning == true then

			posCurrentCameraPosition = octane.vec.add(posCurrentCameraPosition,posPannedCameraPosition)
				print(" posCurrentCameraPosition2",posCurrentCameraPosition[1],posCurrentCameraPosition[2],posCurrentCameraPosition[3])
	
			posCurrentCameraTarget = octane.vec.add(posCurrentCameraTarget, posPannedCameraTarget)
				print(" posCurrentCameraTarget1",posCurrentCameraTarget[1],posCurrentCameraTarget[2],posCurrentCameraTarget[3])

		end


		if blnRotating == true then
			
			if blnPanning == true then

				posCurrentCameraTarget = octane.vec.add(posCurrentCameraPosition, vecViewRotationDirection )
					print(" posCurrentCameraTarget2",posCurrentCameraTarget[1],posCurrentCameraTarget[2],posCurrentCameraTarget[3])

			else

				posCurrentCameraTarget = octane.vec.add(posCurrentCameraTarget, posRotatedCamTarget)
					print(" posCurrentCameraTarget2",posCurrentCameraTarget[1],posCurrentCameraTarget[2],posCurrentCameraTarget[3])

			end

		end

	else

		posCurrentCameraTarget = posInitialCameraTarget
			print(" posCurrentCameraTarget",posCurrentCameraTarget[1],posCurrentCameraTarget[2],posCurrentCameraTarget[3])

	end


    -- Store the new camera Position
	table.insert(tblCameraPositions, posCurrentCameraPosition)


    -- Store the new camera Target
	table.insert(tblCameraTargets, posCurrentCameraTarget)

-- Start Broken
    -- Store the new camera Up
	table.insert(tblCameraUp, vecCurrentCameraUp)
-- End Broken

    -- Store the camera Field Of View (FOV)
	numCurrentFieldOfView = numInitialFieldOfView - (ease(V) * (numInitialFieldOfView - numFinalFieldOfView))
	table.insert(tblFieldOfView, numCurrentFieldOfView)
	print(" Current field of view", numCurrentFieldOfView)


    -- Store the camera Near Clip Depth
	if blnInvertNCD == false then

           		numCurrentNearClipDepth = numCurrentNearClipDepth + ((numFinalNearClipDepth - numInitialNearClipDepth  )/nbFrames)

	else

		numCurrentNearClipDepth = numCurrentNearClipDepth + (( numFinalNearClipDepth - numInitialNearClipDepth  )/nbFrames)

	end


	print(" Current near clip depth", numCurrentNearClipDepth)
	table.insert(tblNearClipDepth, numCurrentNearClipDepth )


	-- Store final positions for multiple camera moves
	posEndCameraPosition	= posCurrentCameraPosition
	posEndCameraTarget	= posCurrentCameraTarget
	numEndFieldOfView	= numCurrentFieldOfView
	numEndNearClipDepth	= numCurrentNearClipDepth

	vecFinalCameraUp = 0

-- Start Broken
	--Reset the vectors to zero for next i
	posCurrentCameraPosition 		= octane.vec.scale(posInitialCameraPosition , 0)
	posCurrentCameraTarget   		= octane.vec.scale(posInitialCameraTarget , 0) 
	vecCurrentCameraUp 		= {0,1,0}
-- Start Broken


    end


   	 -- Animate the camera position
	camNode:getConnectedNode(octane.P_POSITION):setAnimator(octane.A_VALUE, { 0 }, tblCameraPositions, 1 / nbFrames)
	camNode:getConnectedNode(octane.P_TARGET):setAnimator(octane.A_VALUE, { 0 }, tblCameraTargets, 1 / nbFrames)
	camNode:getConnectedNode(octane.P_UP):setAnimator(octane.A_VALUE, { 0 }, tblCameraUp, 1 / nbFrames)

	-- Animate the camera settings
	camNode:getConnectedNode(octane.P_FOV):setAnimator(octane.A_VALUE, { 0 }, tblFieldOfView, 1 / nbFrames)
	camNode:getConnectedNode(octane.P_NEAR_CLIP_DEPTH):setAnimator(octane.A_VALUE, { 0 }, tblNearClipDepth, 1 / nbFrames)
	
	print("--------------------------------------------------------------------------------------------------------")
	print("End of Camera Move "..(numCameraMove+1))
	print(" End Camera Position: ",posEndCameraPosition[1],posEndCameraPosition[2],posEndCameraPosition[3])
	print(" End Camera Target: ", posEndCameraTarget[1],posEndCameraTarget[2],posEndCameraTarget[3])
	print(" End FOV: ",numEndFieldOfView)
	print(" End Near Clip Depth: ",numEndNearClipDepth)
	print("--------------------------------------------------------------------------------------------------------")
  

end


-- Calculate the new sun position for each frame
local function setHourAnimator(sunDirNode,numSolarStartHour, numSolarEndHour, nbFrames)

    -- calculate the new hour for each frame
    local hours = {}

    for i=0,nbFrames-1 do

        -- calculate the hour for each frame
        local hour = numSolarStartHour + (numSolarEndHour - numSolarStartHour) * (i / nbFrames)

        -- store the new hour
        table.insert(hours, { hour, 0, 0 } )

    end

    -- Animate the hour node connected to the daylight environment
    sunDirNode:getConnectedNode(octane.P_HOUR):setAnimator(octane.A_VALUE, { 0 }, hours, 1 / nbFrames)

end


-- Flag indicating cancellation
IS_CANCELLED  = false


function renderCallback()
	-- check if rendering was cancelled
	if (IS_CANCELLED) then

		octane.render.stop()

		return

	end

end


function startRender(sceneGraph, rtNode, camNode, sunDirNode, path, saveFrames)

	print(sceneGraph, rtNode, camNode, sunDirNode, path)
    
    	-- Clear the cancel flag
    	IS_CANCELLED = false

    	octane.render.setShutterTime(sldMotionBlurExposure:getProperties().value)


    	-- Disable part of the ui except for the cancel button
	defaultLoadButton		:updateProperties{ enable = false }
	defaultSaveButton		:updateProperties{ enable = false }
	defaultResetButton		:updateProperties{ enable = false }

    	sldCameraOrbitLeftRight	:updateProperties{ enable = false }
    	sldCameraOrbitUpDown	:updateProperties{ enable = false }
	sldCameraPanLeftRight	:updateProperties{ enable = false }
	sldCameraPanUpDown	:updateProperties{ enable = false }
    	sldStartAngleOffset		:updateProperties{ enable = false }
	sldTargetLeftRight		:updateProperties{ enable = false }
	sldTargetUpDown		:updateProperties{ enable = false }
   	targetSlider		:updateProperties{ enable = false }
   	sldSamples		:updateProperties{ enable = false }
    	sldDuration		:updateProperties{ enable = false }
    	sldFrameRate		:updateProperties{ enable = false }
    	sldNumberOfFrames	:updateProperties{ enable = false }
	btnPreview		:updateProperties{ enable = false }
	sldPreview		:updateProperties{ enable = false }
    	btnOutputFrames		:updateProperties{ enable = false }
	btnOutputPositions		:updateProperties{ enable = false }
    	sldCameraFowardBack	:updateProperties{ enable = false }
    	sldMotionBlurExposure	:updateProperties{ enable = false }
    	renderTargetDropDown	:updateProperties{ enable = false }
    	stopButton		:updateProperties{ enable = true  }

	sldSolarStartHour		:updateProperties{ enable = false }
	sldSolarEndHour		:updateProperties{ enable = false }

	sldFieldOfView		:updateProperties{ enable = false }
	sldNearClipDepth    	:updateProperties{ enable = false }
	chkInvert			:updateProperties{ enable = false}


    -- Get the presets from the GUI
    	local rotAngle       		= sldCameraOrbitLeftRight:getProperties().value
   	local angRotationOffset    	= sldStartAngleOffset:getProperties().value
    	--local numDistanceToTarget	= targetSlider:getProperties().value
   	local nbFrames		= sldNumberOfFrames:getProperties().value
    	local nbSamples		= sldSamples:getProperties().value
    	local outPath       		= fileEditorFrames:getProperties().text
    	local numSolarStartHour 	= sldSolarStartHour:getProperties().value
    	local numSolarEndHour   	= sldSolarEndHour:getProperties().value


    if saveFrames == false then

      nbSamples = sldPreview:getProperties().value

    end


-- Creates a save path for the current frame
function createSavePath(path, frame)

	local file = octane.file.getFileName(path)

	-- Strip png extension
	file = file:gsub("%.png$", "")

	-- Split file into prefix, sequence number and suffix
	-- Make sure the sequence number is in the final file name part
	local prefix, sequenceMatch, suffix = file:match("(.-)(%d+)([^\\/]*)$")
        
	-- If the file name doesn't contain a sequence number, the match fails
	-- So just assume it is all prefix.
	if sequenceMatch == nil then

        		prefix = file
        		sequenceMatch = "0000"
        		suffix = ""

	end
	
	-- Pattern for string.format.
	seqPattern = "%0"..sequenceMatch:len().."d"

	-- Add png extension
	suffix = suffix..".png"
	
	-- Return the path to the output file
	return octane.file.getParentDirectory(path).."/"..prefix.." - "..seqPattern:format(frame)..suffix

end


-- Set up the animator for the camera
setCamAnimator(camNode, rotAngle, angRotationOffset, numDistanceToTarget, nbFrames)

-- Set up the animator for the hour
if sunDirNode ~= nil then
    	setHourAnimator(sunDirNode, numSolarStartHour, numSolarEndHour, nbFrames)
end

-- Start rendering out each frame
local currentTime = 0


for frame = 1, nbFrames do

	-- Set the time in the scene 
	sceneGraph:updateTime(currentTime)
        
	-- Update the progress bar
	progressBar:updateProperties{ text = string.format("rendering frame %d", frame) }

	-- Fire up the render engine
	octane.render.start
	{
		renderTargetNode	= rtNode,
		maxSamples	= nbSamples,
		callback		= renderCallback,
	}
        

        -- Break out if we're cancelled and set it in the progress bar
        if IS_CANCELLED then

            progressBar:updateProperties{ progress = 0, text = "cancelled" } 
            break 

        end


        -- Save the current frame
	if saveFrames == true then

		if blnMultipleMoves == true then
			
			-- Strip png extension
			local filename = OUT_PATH:gsub("%.png$", "")

			-- Split into parts
			local prefix, sequenceMatch, suffix = filename:match("(.-)(%d+)([^\\/]*)$")

			if sequenceMatch == nil then
        				prefix = filename
			end

			-- Pattern for string.format.
			local seqPattern = "%0" .. tostring(nbFrames):len().."d"
			outputfilename = prefix.." - ".."Move "..(numCameraMove).."." .. "Frame " .. seqPattern:format(frame)..".png"
        			octane.render.saveImage(outputfilename, octane.render.imageType.PNG8)

			print("Output frame to:",outputfilename)


		elseif blnMultipleMoves == false then

        			local out = createSavePath(path, frame)
        			octane.render.saveImage(out, octane.render.imageType.PNG8)


		end


	end


        -- Update the time for the next frame
        	currentTime = frame * (1 / nbFrames)


        -- Update the progress bar
        	progressBar:updateProperties{ progress = frame / nbFrames }


    end


    -- Enable part of the ui except for the cancel button

	defaultLoadButton 		:updateProperties{ enable = true }
	defaultSaveButton 		:updateProperties{ enable = true }
	defaultResetButton 		:updateProperties{ enable = true }
	sldCameraOrbitLeftRight	:updateProperties{ enable = true }
	sldCameraOrbitUpDown	:updateProperties{ enable = true }
	sldCameraPanLeftRight	:updateProperties{ enable = true }
	sldCameraPanUpDown	:updateProperties{ enable = true }
	sldStartAngleOffset		:updateProperties{ enable = true }
	targetSlider           		:updateProperties{ enable = true }
	sldTargetLeftRight		:updateProperties{ enable = true }
	sldTargetUpDown		:updateProperties{ enable = true}
	sldSamples		:updateProperties{ enable = true }
	sldDuration		:updateProperties{ enable = true }
	sldFrameRate		:updateProperties{ enable = true }
	sldNumberOfFrames	:updateProperties{ enable = true }
	btnOutputFrames		:updateProperties{ enable = true }
	btnOutputPositions		:updateProperties{ enable = true }
	btnPreview		:updateProperties{ enable = true }
	sldPreview		:updateProperties{ enable = true }
	sldCameraFowardBack	:updateProperties{ enable = true }
	sldMotionBlurExposure	:updateProperties{ enable = true }
	renderTargetDropDown	:updateProperties{ enable = true }
	stopButton		:updateProperties{ enable = false}


	sldFieldOfView		:updateProperties{ enable = true}
	sldNearClipDepth    	:updateProperties{ enable = true}	
	chkInvert			:updateProperties{ enable = true}


	if copySunDir == nil then

		sldSolarStartHour		:updateProperties{ enable = false }
		sldSolarEndHour		:updateProperties{ enable = false }

	else

		sldSolarStartHour		:updateProperties{ enable = true }
		sldSolarEndHour		:updateProperties{ enable = true }

	end

    -- Update the progress bar
    progressBar:updateProperties{ progress = 0, text = "finished" }


end


function cancelRender()

    IS_CANCELLED = true
    octane.render.callbackStop()
    octane.render.clear()

end


function round(num, idp)

  local mult = 10^(idp or 0)
  return math.floor(num * mult + 0.5) / mult

end



function initGui(camNode)

    -- Set the initial target distance in the distance slider

	local fov 		= camNode:getPinValue(octane.P_FOV)
	local ncd		= camNode:getPinValue(octane.P_NEAR_CLIP_DEPTH)
   	local target   	= camNode:getPinValue(octane.P_TARGET)
    	local position 	= camNode:getPinValue(octane.P_POSITION)
    	local viewDir  	= octane.vec.sub(target, position)
    	local dtt   		= octane.vec.length(viewDir)
	dtt 		= round(dtt,3)


	lblInitialFieldOfView:updateProperties{ text = "Field of view: " .. tostring(fov)}	
	lblInitialNearClip:updateProperties{ text = "Near clip depth: " .. tostring(ncd)}	
	lblInitialDistanceToTarget:updateProperties{ text = "Distance to target: " .. tostring(dtt)}

	--sldFieldOfView:updateProperties{ value = fov }
	--sldNearClipDepth:updateProperties{ value = tonumber(lblInitialNearClip.text) }

    -- Disable stop button
    -- stopButton:updateProperties{ enable = false }

end


